/*
	Piotr Poski, GIBE
	Sposb uycia random_sample, random_sample_n
	Przygotowane na podstawie:
	http://www.sgi.com/tech/stl/random_sample.html
	http://www.sgi.com/tech/stl/random_sample_n.html

	Obie funkcje random_sample i random_sample_n pochodz z rozszerzenia SGI
	i nie s czsci standardu C++.

	*** random_sample ***
	random_sample przeadowywuje dwie funkcje:
	
	template <class InputIterator, class RandomAccessIterator>
	RandomAccessIterator random_sample(InputIterator first, 
										InputIterator last,
    									RandomAccessIterator ofirst, 
										RandomAccessIterator olast) 

	template <class InputIterator, class RandomAccessIterator,
  	class RandomNumberGenerator>
	RandomAccessIterator random_sample(InputIterator first, 
										InputIterator last,
    									RandomAccessIterator ofirst, 
										RandomAccessIterator olast,
    									RandomNumberGenerator& rand) 

	Opis dziaania:
	Losowo kopiuje n elementw z zakresu [first, last) do zakresu [ofirst, olast),
	gdzie n = min(last-first, olast-ofirst). Kady element ze zbioru wejciowego 
	moe pojawi si tylko raz w zbiorze wyjciowym, elementy s wybierane z jednakowym 
	prawdopodobiestwem. Elementy na wyjciu mog pojawi si w dowolnej kolejnoci. 
	Pierwsza wersja random_sample kozysta z wewntrznego generatora liczb losowych, 
	druga wersja pobiera jako argument funktor(obiekt funkcj) zdefiniowanego przez 
	uytkownika generatora. Elementem zwracanym przez random_sample jest element ofirst+n.
	
	Parametry:
	InputIterator - iterator ktry moe by inkrementowany by uzyska dostp do nastpnego obiektu
	RandomAccessIterator - iterator ktry moe by inkrementowany i dekrementownay	
						   (kroki mog by rnej dugoci)
	RandomNumberGenerator - obiekt-funkcja, ktry pozwala na genrowanie liczb cakowitych z 
							zakresu [0, N) np.: jeeli f jest Random Number Generator to f(N) zwraca
							liczb cakowit mniejsz N i wieksz lub rwn zero

	*** random_sample_n ***
	random_sample_n przadowywuje dwie funkcje:
	
	template <class ForwardIterator, class OutputIterator, class Distance>
	OutputIterator random_sample_n(ForwardIterator first, 
									ForwardIterator last,
                               		OutputIterator out, 
									Distance n)

	template <class ForwardIterator, class OutputIterator, class Distance,
    class RandomNumberGenerator>
	OutputIterator random_sample_n(ForwardIterator first, 
									ForwardIterator last,
                               		OutputIterator out,
									Distance n,
                               		RandomNumberGenerator& rand)

	Opis dziaania:
	Losowo kopiuje m elementw z zakresu [first, last) do zakresu [out, out+m), gdzie
	m = min(last-first, n). Kady element wejciowy moe pojawi si na wyjciu dokadnie 
	raz, elementy wybierane s z jednakowym	prawdopodobiestwem. Elementy na wyjciu s losowe 
	ale zachowuj porzdek tak jak byy ustawione na wejciu (wana rnica midzy random_sample !).  
	Pierwsza wersja random_sample_n kozysta z wewntrznego generatora liczb losowych, 
	druga wersja pobiera jako argument funktor(obiekt funkcj) zdefiniowanego przez 
	uytkownika generatora. Funkcja zwraca element out+m.

	Parametry:
	ForwardIterator - iterator ktry pozwala tylko pobiera nastpne elementy
	OutputIterator - iterator w ktrym mona zapisa warto do wskazywanej pozycji i przesun si
					 do wskazywania na nastpny element ale nie mona odczyta wartoci i cofn si.
	Distance - warto cakowita wyznaczajca liczb elementw na wyjciu
	RandomNumberGenerator - funktor (zachowanie takie samo jak dal random_sample), warto
							Distance jest jego argumentem


	Zoono obu algorytmw jest liniowa.
*/		
#include <iostream>
#include <ext/algorithm>
#include <time.h>
#include <vector>
#include <iterator>

using namespace std;

/*
	Funktor - obiekt ktry moe by wywoany tak jakby by funkcj.
	Obiekt ten definiuje operator().
*/
struct RandomNumberGenerator
{
	int operator()(int r)
	{
		return rand()%r;	
	}
};

int main () 
{
	// inicjacja zarodka generatora
	srand(time(NULL));
	int i;
	const int N = 10;
	const int n = 6;
	int A[] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
	int B[n];
	
	/* random_sample */ 
	cout << "random_sample\n";
	int *ret_val;	// wskanik na warto zwracan
	ret_val = __gnu_cxx::random_sample(A, A+N, B, B+n); // przykad dziaania
	cout << *(ret_val-1) << endl;	// warto zwrcona
	for(i = 0; i < n; ++i)
		cout << B[i] << " ";		// wartoci na wyjciu
	cout << endl;	

	RandomNumberGenerator rng;     	// funktor 
	cout << rng(13) << endl;		// przykad uycia funktora
	__gnu_cxx::random_sample(A, A+N, B, B+n, rng);		// wersja z genratorem zewntrznym 
	copy(B, B + n, ostream_iterator<int>(cout, " "));  	// wypisanie z uyciem operatora ostream_iterator
	cout << endl;										// i funkcji copy

	/* random_sample_n */
	cout << "random_sample_n\n";
	vector<int> vec;				// wektor do zapisu elementw wyjciowych
	vector<int>::const_iterator it;
	insert_iterator<vector<int> > ii(vec, vec.begin()); // insert_iterator do wstawiania elementw do wektora
	__gnu_cxx::random_sample_n(A, A+N, ii, n);		// przykad dziaania funkcji
	for(it = vec.begin(); it != vec.end(); ++it)
		cout << *it << " ";
	cout << endl;
	
	const int M = 6;
	const int m = 3;
	string AA[] = {"ola", "ala", "ela", "ele", "alo", "olo"}; // przykad dziaania funkcji na tablicy ze stringami
	// funkcja z generatorem zewnetrznym i wynikiem dziaania przekazywanym od razu na standardowe wyjcie	
	__gnu_cxx::random_sample_n(AA, AA+M, ostream_iterator<string>(cout ," "), m, rng); 
	cout << endl;

  return 0;
}
